# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from functools import update_wrapper

from django.contrib import admin
from django.contrib.admin.utils import quote
from django.contrib.admin.views.main import ChangeList
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse

from simple_log.utils import get_log_model


class SimpleLogChangeList(ChangeList):
    def url_for_result(self, result):
        history_for_model = getattr(self.model_admin,
                                    'history_for_model', None)
        if history_for_model:
            object_opts = history_for_model._meta
            pk = getattr(result, self.pk_attname)
            object_pk = getattr(result, 'object_id')
            return reverse(
                'admin:%s_%s_history_detail' % (object_opts.app_label,
                                                object_opts.model_name),
                args=(quote(object_pk), quote(pk),),
                current_app=self.model_admin.admin_site.name
            )
        return super(SimpleLogChangeList, self).url_for_result(result)


class SimpleLogModelAdmin(admin.ModelAdmin):
    list_display = ('object_repr', 'content_type', 'action_flag',
                    'action_time', 'user_repr', 'user_ip')
    list_filter = ('action_flag', 'content_type')
    search_fields = ('object_repr', 'user_repr', 'user_ip')
    change_form_template = 'simple_log/admin/detail.html'

    def __init__(self, model, admin_site, change_list_template=None,
                 history_for_model=None, history_for_object=None):
        super(SimpleLogModelAdmin, self).__init__(model, admin_site)
        self.history_for_model = history_for_model
        self.history_for_object = history_for_object
        if change_list_template:
            self.change_list_template = change_list_template

    def has_delete_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return False

    def get_actions(self, request):
        return []

    def get_list_filter(self, request):
        ret = super(SimpleLogModelAdmin, self).get_list_filter(request)
        if self.history_for_model:
            ret = [x for x in ret if x != 'content_type']
        return ret

    def get_queryset(self, request):
        qs = super(SimpleLogModelAdmin, self).get_queryset(request)
        if self.history_for_model:
            ct = ContentType.objects.get_for_model(self.history_for_model)
            qs = qs.filter(content_type=ct)
            if self.history_for_object:
                qs = qs.filter(object_id=self.history_for_object)
        return qs.select_related('content_type', 'user')

    def get_changelist(self, request, **kwargs):
        return SimpleLogChangeList


class HistoryModelAdmin(admin.ModelAdmin):
    change_list_template = 'simple_log/admin/change_list.html'
    history_change_list_template = 'simple_log/admin/history_change_list.html'

    def get_urls(self):
        from django.conf.urls import url

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)

            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name

        urlpatterns = super(HistoryModelAdmin, self).get_urls()
        custom_urlpatterns = [
            url(r'^history/$', wrap(self.model_history_view),
                name='%s_%s_model_history' % info),
            url(r'^(.+)/history/(.+)/$', wrap(self.history_detail_view),
                name='%s_%s_history_detail' % info),
        ]
        return custom_urlpatterns + urlpatterns

    def get_simple_log_admin_model(self, model=None, object_id=None):
        simple_log_model = get_log_model()
        model_admin = admin.site._registry.get(simple_log_model,
                                               SimpleLogModelAdmin)
        return model_admin.__class__(
            simple_log_model,
            self.admin_site,
            self.history_change_list_template,
            model,
            object_id
        )

    def model_history_view(self, request, extra_context=None):
        if not self.has_change_permission(request, None):
            raise PermissionDenied

        admin_model = self.get_simple_log_admin_model(model=self.model)
        return admin_model.changelist_view(
            request,
            extra_context={
                'history_model_opts': self.model._meta
            }
        )

    def history_view(self, request, object_id, extra_context=None):
        if not self.has_change_permission(request):
            raise PermissionDenied

        obj = self.get_object(request, object_id, extra_context)

        admin_model = self.get_simple_log_admin_model(
            model=self.model,
            object_id=object_id
        )
        return admin_model.changelist_view(
            request,
            extra_context={
                'history_model_opts': self.model._meta,
                'history_object': obj
            }
        )

    def history_detail_view(self, request, object_id, history_id,
                            extra_context=None):
        if not self.has_change_permission(request):
            raise PermissionDenied

        obj = self.get_object(request, object_id, extra_context)
        admin_model = self.get_simple_log_admin_model(
            model=self.model,
            object_id=object_id
        )
        return admin_model.changeform_view(
            request,
            history_id,
            extra_context={
                'history_model_opts': self.model._meta,
                'history_object': obj
            }
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations
from django.conf import settings

from simple_log.fields import SimpleManyToManyField


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.SIMPLE_LOG_MODEL),
        ('simple_log', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='simplelog',
            name='related_logs',
            field=SimpleManyToManyField(to='self', verbose_name='related log', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from threading import local

from contextlib import contextmanager
from django.apps import apps as django_apps
from django.core.exceptions import ImproperlyConfigured
from django.utils import lru_cache, six
from django.utils.encoding import force_text
from django.utils.module_loading import import_string

from simple_log.conf import settings

__all__ = ['set_thread_variable', 'get_thread_variable', 'del_thread_variable',
           'get_log_model', 'get_current_user', 'get_current_request',
           'get_serializer', 'disable_logging', 'get_model_list',
           'disable_related', 'get_obj_repr']


_thread_locals = local()


def set_thread_variable(key, val):
    setattr(_thread_locals, key, val)


def get_thread_variable(key, default=None):
    return getattr(_thread_locals, key, default)


def del_thread_variable(key):
    if hasattr(_thread_locals, key):
        return delattr(_thread_locals, key)


def check_log_model(model):
    from simple_log.models import SimpleLogAbstractBase
    if not issubclass(model, SimpleLogAbstractBase):
        raise ImproperlyConfigured('Log model should be subclass of '
                                   'SimpleLogAbstract.')
    return model


@lru_cache.lru_cache()
def get_log_model():
    try:
        return check_log_model(django_apps.get_model(settings.MODEL))
    except (ValueError, AttributeError):
        raise ImproperlyConfigured("SIMPLE_LOG_MODEL must be of the form "
                                   "'app_label.model_name'")
    except LookupError:
        raise ImproperlyConfigured(
            "SIMPLE_LOG_MODEL refers to model '%s' "
            "that has not been installed" % settings.MODEL
        )


@lru_cache.lru_cache(maxsize=None)
def get_serializer(model=None):
    if hasattr(model, 'simple_log_serializer'):
        serializer = model.simple_log_serializer
    else:
        serializer = settings.MODEL_SERIALIZER
    if isinstance(serializer, six.string_types):
        return import_string(serializer)
    return serializer


def get_current_request_default():
    return get_thread_variable('request')


@lru_cache.lru_cache(maxsize=None)
def _get_current_request():
    return import_string(settings.GET_CURRENT_REQUEST)


get_current_request = _get_current_request()


def get_current_user():
    request = get_current_request()
    if request:
        return getattr(request, 'user', None)


@lru_cache.lru_cache()
def get_fields(klass):
    fields = klass._meta.get_fields()
    if hasattr(klass, 'simple_log_fields'):
        fields = [f for f in fields if f.name in klass.simple_log_fields]
    elif hasattr(klass, 'simple_log_exclude_fields'):
        fields = [f for f in fields
                  if f.name not in klass.simple_log_exclude_fields]
    else:
        fields = [f for f in fields
                  if f.name not in settings.EXCLUDE_FIELD_LIST]
    return [f for f in fields if f.concrete or
            (settings.SAVE_ONE_TO_MANY and f.one_to_many and f.related_name)]


def get_label(m):
    try:
        return m._meta.label
    except AttributeError:
        return '{}.{}'.format(m._meta.app_label, m._meta.object_name)


@lru_cache.lru_cache(maxsize=None)
def get_model_list():
    from simple_log.models import SimpleLogAbstractBase
    model_list = [m for m in django_apps.get_models()
                  if not issubclass(m, SimpleLogAbstractBase)]
    if settings.MODEL_LIST:
        model_list = [m for m in model_list
                      if get_label(m) in settings.MODEL_LIST]
    if settings.EXCLUDE_MODEL_LIST:
        model_list = [m for m in model_list
                      if get_label(m) not in settings.EXCLUDE_MODEL_LIST]
    return model_list


@lru_cache.lru_cache()
def get_related_models(model):
    return [x.related_model for x in model._meta.get_fields()
            if x.many_to_one or x.one_to_one]


@contextmanager
def disable_logging():
    set_thread_variable('disable_logging', True)
    try:
        yield
    finally:
        del_thread_variable('disable_logging')


@contextmanager
def disable_related():
    set_thread_variable('disable_related', True)
    try:
        yield
    finally:
        del_thread_variable('disable_related')


def get_obj_repr(obj):
    if hasattr(obj, 'simple_log_repr'):
        return force_text(obj.simple_log_repr())
    return force_text(obj)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.template import Library
from django.utils import six
from django.utils.encoding import force_text

register = Library()


@register.assignment_tag()
def get_type(value):
    if isinstance(value, six.string_types):
        return 'str'
    if value is None:
        return 'None'
    if isinstance(value, dict):
        return 'dict'
    if isinstance(value, bool):
        return 'bool'
    if isinstance(value, list):
        return 'list'
    if isinstance(value, int):
        return 'int'
    return force_text(type(value))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.apps import apps
from django.core.management.base import BaseCommand

from simple_log.utils import get_fields, get_model_list, get_label


class Command(BaseCommand):
    help = 'View all registered models, mark which is tracking'

    def add_arguments(self, parser):
        parser.add_argument(
            '-f', '--with_fields',
            action='store_true',
            default=False,
            help='Show which fields is tracking for every model'
        )

    def handle(self, *args, **options):
        self.stdout.write('')
        for model in apps.get_models():
            tracking = model in get_model_list()
            if not tracking and options['with_fields']:
                continue
            prefix = '[{}] '.format('+' if tracking else '-')
            self.stdout.write(prefix + get_label(model))
            if tracking and options['with_fields']:
                fields = get_fields(model)
                self.stdout.write(
                    '    - ' + '\n    - '.join([x.name for x in fields]),
                )
        self.stdout.write('')



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-06 17:46
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone

from simple_log.fields import SimpleJSONField


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='SimpleLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('action_time', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='action time')),
                ('user_repr', models.CharField(blank=True, max_length=1000, verbose_name='user repr')),
                ('user_ip', models.GenericIPAddressField(null=True, verbose_name='IP address')),
                ('object_id', models.TextField(blank=True, null=True, verbose_name='object id')),
                ('object_repr', models.CharField(max_length=1000, verbose_name='object repr')),
                ('action_flag', models.PositiveSmallIntegerField(choices=[(1, 'added'), (2, 'changed'), (3, 'deleted')], verbose_name='action flag')),
                ('old', SimpleJSONField(null=True, verbose_name='old values')),
                ('new', SimpleJSONField(null=True, verbose_name='new values')),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='contenttypes.ContentType', verbose_name='content type')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'verbose_name': 'log entry',
                'verbose_name_plural': 'logs entries',
                'ordering': ('-action_time',),
                'abstract': False,
                'swappable': 'SIMPLE_LOG_MODEL',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from distutils.core import setup

from setuptools import find_packages


def readme():
    with open('README.rst') as f:
        return f.read()


setup(
    name='django-simple-log',
    version='0.1.18',
    description='Logging django models changes.',
    long_description=readme(),
    keywords=['django', 'log', 'audit', 'history'],
    author='Grigory Mishchenko',
    author_email='grishkokot@gmail.com',
    url='https://github.com/kindlycat/django-simple-log/',
    packages=find_packages(exclude=('manage', 'tests', 'tests.*')),
    include_package_data=True,
    install_requires=['Django>=1.8'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'License :: OSI Approved :: BSD License',
    ],
)
